@@ -4,7 +4,7 @@
 #if !defined(LTTNG_TRACE_RCU_H) || defined(TRACE_HEADER_MULTI_READ)
 #define LTTNG_TRACE_RCU_H
 
-#include "../../../probes/lttng-tracepoint-event.h"
+#include <probes/lttng-tracepoint-event.h>
 #include <linux/version.h>
 
 /*
@@ -624,4 +624,4 @@ LTTNG_TRACEPOINT_EVENT(rcu_barrier,
 #endif /* LTTNG_TRACE_RCU_H */
 
 /* This part must be outside protection */
-#include "../../../probes/define_trace.h"
+#include <probes/define_trace.h>
